@@ -3,20 +3,43 @@ use crate::fields::{
     Md5Checksum, MultiArch, Priority, Sha1Checksum, Sha256Checksum, Sha512Checksum,
 };
 use crate::lossless::relations::Relations;
+use rowan::ast::AstNode;
 
 /// A source package in the APT package manager.
+#[derive(Debug, Clone, PartialEq, Eq)]
 pub struct Source(deb822_lossless::Paragraph);
 
 #[cfg(feature = "python-debian")]
-impl pyo3::ToPyObject for Source {
-    fn to_object(&self, py: pyo3::Python) -> pyo3::PyObject {
+impl<'py> pyo3::IntoPyObject<'py> for Source {
+    type Target = pyo3::PyAny;
+    type Output = pyo3::Bound<'py, Self::Target>;
+    type Error = pyo3::PyErr;
+
+    fn into_pyobject(self, py: pyo3::Python<'py>) -> Result<Self::Output, Self::Error> {
         use pyo3::prelude::*;
-        let d = self.0.to_object(py);
+        let d = self.0.into_pyobject(py)?;
 
-        let m = py.import_bound("debian.deb822").unwrap();
-        let cls = m.getattr("Sources").unwrap();
+        let m = py.import("debian.deb822")?;
+        let cls = m.getattr("Sources")?;
 
-        cls.call1((d,)).unwrap().to_object(py)
+        cls.call1((d,))
+    }
+}
+
+#[cfg(feature = "python-debian")]
+impl<'a, 'py> pyo3::IntoPyObject<'py> for &'a Source {
+    type Target = pyo3::PyAny;
+    type Output = pyo3::Bound<'py, Self::Target>;
+    type Error = pyo3::PyErr;
+
+    fn into_pyobject(self, py: pyo3::Python<'py>) -> Result<Self::Output, Self::Error> {
+        use pyo3::prelude::*;
+        let d = (&self.0).into_pyobject(py)?;
+
+        let m = py.import("debian.deb822")?;
+        let cls = m.getattr("Sources")?;
+
+        cls.call1((d,))
     }
 }
 
@@ -46,6 +69,32 @@ impl Source {
         Self(deb822_lossless::Paragraph::new())
     }
 
+    /// Parse source package text, returning a Parse result
+    ///
+    /// Note: This expects a single paragraph, not a full deb822 document
+    pub fn parse(text: &str) -> deb822_lossless::Parse<Source> {
+        // We need to transform Parse<Deb822> to Parse<Source>
+        // Since Source wraps a single Paragraph, we need custom logic
+        let deb822_parse = deb822_lossless::Deb822::parse(text);
+
+        // For now, we'll use the same green node but the cast will extract the first paragraph
+        let green = deb822_parse.green().clone();
+        let mut errors = deb822_parse.errors().to_vec();
+
+        // Check if there's exactly one paragraph
+        if errors.is_empty() {
+            let deb822 = deb822_parse.tree();
+            let paragraph_count = deb822.paragraphs().count();
+            if paragraph_count == 0 {
+                errors.push("No paragraphs found".to_string());
+            } else if paragraph_count > 1 {
+                errors.push("Multiple paragraphs found, expected one".to_string());
+            }
+        }
+
+        deb822_lossless::Parse::new(green, errors)
+    }
+
     /// Get the source name
     pub fn package(&self) -> Option<String> {
         self.0.get("Package").map(|s| s.to_string())
@@ -447,23 +496,70 @@ impl std::str::FromStr for Source {
     type Err = deb822_lossless::ParseError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        Ok(Self(s.parse()?))
+        Source::parse(s).to_result()
+    }
+}
+
+impl AstNode for Source {
+    type Language = deb822_lossless::Lang;
+
+    fn can_cast(kind: <Self::Language as rowan::Language>::Kind) -> bool {
+        // Source can be cast from either a Paragraph or a Deb822 (taking first paragraph)
+        deb822_lossless::Paragraph::can_cast(kind) || deb822_lossless::Deb822::can_cast(kind)
+    }
+
+    fn cast(syntax: rowan::SyntaxNode<Self::Language>) -> Option<Self> {
+        // Try to cast as Paragraph first
+        if let Some(para) = deb822_lossless::Paragraph::cast(syntax.clone()) {
+            Some(Source(para))
+        } else if let Some(deb822) = deb822_lossless::Deb822::cast(syntax) {
+            // If it's a Deb822, take the first paragraph
+            deb822.paragraphs().next().map(Source)
+        } else {
+            None
+        }
+    }
+
+    fn syntax(&self) -> &rowan::SyntaxNode<Self::Language> {
+        self.0.syntax()
     }
 }
 
 /// A package in the APT package manager.
+#[derive(Debug, Clone, PartialEq, Eq)]
 pub struct Package(deb822_lossless::Paragraph);
 
 #[cfg(feature = "python-debian")]
-impl pyo3::ToPyObject for Package {
-    fn to_object(&self, py: pyo3::Python) -> pyo3::PyObject {
+impl<'py> pyo3::IntoPyObject<'py> for Package {
+    type Target = pyo3::PyAny;
+    type Output = pyo3::Bound<'py, Self::Target>;
+    type Error = pyo3::PyErr;
+
+    fn into_pyobject(self, py: pyo3::Python<'py>) -> Result<Self::Output, Self::Error> {
         use pyo3::prelude::*;
-        let d = self.0.to_object(py);
+        let d = self.0.into_pyobject(py)?;
 
-        let m = py.import_bound("debian.deb822").unwrap();
-        let cls = m.getattr("Packages").unwrap();
+        let m = py.import("debian.deb822")?;
+        let cls = m.getattr("Packages")?;
 
-        cls.call1((d,)).unwrap().to_object(py)
+        cls.call1((d,))
+    }
+}
+
+#[cfg(feature = "python-debian")]
+impl<'a, 'py> pyo3::IntoPyObject<'py> for &'a Package {
+    type Target = pyo3::PyAny;
+    type Output = pyo3::Bound<'py, Self::Target>;
+    type Error = pyo3::PyErr;
+
+    fn into_pyobject(self, py: pyo3::Python<'py>) -> Result<Self::Output, Self::Error> {
+        use pyo3::prelude::*;
+        let d = (&self.0).into_pyobject(py)?;
+
+        let m = py.import("debian.deb822")?;
+        let cls = m.getattr("Packages")?;
+
+        cls.call1((d,))
     }
 }
 
@@ -481,6 +577,28 @@ impl Package {
         Self(paragraph)
     }
 
+    /// Parse package text, returning a Parse result
+    ///
+    /// Note: This expects a single paragraph, not a full deb822 document
+    pub fn parse(text: &str) -> deb822_lossless::Parse<Package> {
+        let deb822_parse = deb822_lossless::Deb822::parse(text);
+        let green = deb822_parse.green().clone();
+        let mut errors = deb822_parse.errors().to_vec();
+
+        // Check if there's exactly one paragraph
+        if errors.is_empty() {
+            let deb822 = deb822_parse.tree();
+            let paragraph_count = deb822.paragraphs().count();
+            if paragraph_count == 0 {
+                errors.push("No paragraphs found".to_string());
+            } else if paragraph_count > 1 {
+                errors.push("Multiple paragraphs found, expected one".to_string());
+            }
+        }
+
+        deb822_lossless::Parse::new(green, errors)
+    }
+
     /// Get the name of the package.
     pub fn name(&self) -> Option<String> {
         self.0.get("Package").map(|s| s.to_string())
@@ -748,7 +866,29 @@ impl std::str::FromStr for Package {
     type Err = deb822_lossless::ParseError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        Ok(Self(s.parse()?))
+        Package::parse(s).to_result()
+    }
+}
+
+impl AstNode for Package {
+    type Language = deb822_lossless::Lang;
+
+    fn can_cast(kind: <Self::Language as rowan::Language>::Kind) -> bool {
+        deb822_lossless::Paragraph::can_cast(kind) || deb822_lossless::Deb822::can_cast(kind)
+    }
+
+    fn cast(syntax: rowan::SyntaxNode<Self::Language>) -> Option<Self> {
+        if let Some(para) = deb822_lossless::Paragraph::cast(syntax.clone()) {
+            Some(Package(para))
+        } else if let Some(deb822) = deb822_lossless::Deb822::cast(syntax) {
+            deb822.paragraphs().next().map(Package)
+        } else {
+            None
+        }
+    }
+
+    fn syntax(&self) -> &rowan::SyntaxNode<Self::Language> {
+        self.0.syntax()
     }
 }
 
@@ -756,15 +896,36 @@ impl std::str::FromStr for Package {
 pub struct Release(deb822_lossless::Paragraph);
 
 #[cfg(feature = "python-debian")]
-impl pyo3::ToPyObject for Release {
-    fn to_object(&self, py: pyo3::Python) -> pyo3::PyObject {
+impl<'py> pyo3::IntoPyObject<'py> for Release {
+    type Target = pyo3::PyAny;
+    type Output = pyo3::Bound<'py, Self::Target>;
+    type Error = pyo3::PyErr;
+
+    fn into_pyobject(self, py: pyo3::Python<'py>) -> Result<Self::Output, Self::Error> {
         use pyo3::prelude::*;
-        let d = self.0.to_object(py);
+        let d = self.0.into_pyobject(py)?;
 
-        let m = py.import_bound("debian.deb822").unwrap();
-        let cls = m.getattr("Release").unwrap();
+        let m = py.import("debian.deb822")?;
+        let cls = m.getattr("Release")?;
 
-        cls.call1((d,)).unwrap().to_object(py)
+        cls.call1((d,))
+    }
+}
+
+#[cfg(feature = "python-debian")]
+impl<'a, 'py> pyo3::IntoPyObject<'py> for &'a Release {
+    type Target = pyo3::PyAny;
+    type Output = pyo3::Bound<'py, Self::Target>;
+    type Error = pyo3::PyErr;
+
+    fn into_pyobject(self, py: pyo3::Python<'py>) -> Result<Self::Output, Self::Error> {
+        use pyo3::prelude::*;
+        let d = (&self.0).into_pyobject(py)?;
+
+        let m = py.import("debian.deb822")?;
+        let cls = m.getattr("Release")?;
+
+        cls.call1((d,))
     }
 }
 
@@ -782,6 +943,28 @@ impl Release {
         Self(paragraph)
     }
 
+    /// Parse release text, returning a Parse result
+    ///
+    /// Note: This expects a single paragraph, not a full deb822 document
+    pub fn parse(text: &str) -> deb822_lossless::Parse<Release> {
+        let deb822_parse = deb822_lossless::Deb822::parse(text);
+        let green = deb822_parse.green().clone();
+        let mut errors = deb822_parse.errors().to_vec();
+
+        // Check if there's exactly one paragraph
+        if errors.is_empty() {
+            let deb822 = deb822_parse.tree();
+            let paragraph_count = deb822.paragraphs().count();
+            if paragraph_count == 0 {
+                errors.push("No paragraphs found".to_string());
+            } else if paragraph_count > 1 {
+                errors.push("Multiple paragraphs found, expected one".to_string());
+            }
+        }
+
+        deb822_lossless::Parse::new(green, errors)
+    }
+
     /// Get the origin of the release
     pub fn origin(&self) -> Option<String> {
         self.0.get("Origin").map(|s| s.to_string())
@@ -1041,7 +1224,29 @@ impl std::str::FromStr for Release {
     type Err = deb822_lossless::ParseError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        Ok(Self(s.parse()?))
+        Release::parse(s).to_result()
+    }
+}
+
+impl AstNode for Release {
+    type Language = deb822_lossless::Lang;
+
+    fn can_cast(kind: <Self::Language as rowan::Language>::Kind) -> bool {
+        deb822_lossless::Paragraph::can_cast(kind) || deb822_lossless::Deb822::can_cast(kind)
+    }
+
+    fn cast(syntax: rowan::SyntaxNode<Self::Language>) -> Option<Self> {
+        if let Some(para) = deb822_lossless::Paragraph::cast(syntax.clone()) {
+            Some(Release(para))
+        } else if let Some(deb822) = deb822_lossless::Deb822::cast(syntax) {
+            deb822.paragraphs().next().map(Release)
+        } else {
+            None
+        }
+    }
+
+    fn syntax(&self) -> &rowan::SyntaxNode<Self::Language> {
+        self.0.syntax()
     }
 }
 
@@ -1285,4 +1490,90 @@ Multi-Arch: same
         );
         assert_eq!(318, release.checksums_md5().len());
     }
+
+    #[test]
+    fn test_source_vcs_arch() {
+        let s = r#"Package: foo
+Version: 1.0
+Vcs-Arch: https://example.com/arch/repo
+"#;
+        let p: super::Source = s.parse().unwrap();
+        assert_eq!(
+            p.vcs_arch(),
+            Some("https://example.com/arch/repo".to_string())
+        );
+    }
+
+    #[test]
+    fn test_source_vcs_cvs() {
+        let s = r#"Package: foo
+Version: 1.0
+Vcs-Cvs: :pserver:anoncvs@example.com:/cvs/repo
+"#;
+        let p: super::Source = s.parse().unwrap();
+        assert_eq!(
+            p.vcs_cvs(),
+            Some(":pserver:anoncvs@example.com:/cvs/repo".to_string())
+        );
+    }
+
+    #[test]
+    fn test_source_set_priority() {
+        let s = r#"Package: foo
+Version: 1.0
+Priority: optional
+"#;
+        let mut p: super::Source = s.parse().unwrap();
+        p.set_priority(super::Priority::Required);
+        assert_eq!(p.priority(), Some(super::Priority::Required));
+    }
+
+    #[test]
+    fn test_release_set_suite() {
+        let s = r#"Origin: Debian
+Label: Debian
+Suite: testing
+"#;
+        let mut release: super::Release = s.parse().unwrap();
+        release.set_suite("unstable");
+        assert_eq!(release.suite(), Some("unstable".to_string()));
+    }
+
+    #[test]
+    fn test_release_codename() {
+        let s = r#"Origin: Debian
+Label: Debian
+Suite: testing
+Codename: trixie
+"#;
+        let release: super::Release = s.parse().unwrap();
+        assert_eq!(release.codename(), Some("trixie".to_string()));
+    }
+
+    #[test]
+    fn test_release_set_checksums_sha512() {
+        let s = r#"Origin: Debian
+Label: Debian
+Suite: testing
+"#;
+        let mut release: super::Release = s.parse().unwrap();
+        let checksums = vec![super::Sha512Checksum {
+            sha512: "abc123".to_string(),
+            size: 1234,
+            filename: "test.deb".to_string(),
+        }];
+        release.set_checksums_sha512(checksums);
+        assert_eq!(release.checksums_sha512().len(), 1);
+    }
+
+    #[test]
+    fn test_package_set_replaces() {
+        let s = r#"Package: foo
+Version: 1.0
+"#;
+        let mut p: super::Package = s.parse().unwrap();
+        let relations: Relations = "bar (>= 1.0.0)".parse().unwrap();
+        p.set_replaces(relations);
+        assert_eq!(p.replaces(), Some("bar (>= 1.0.0)".parse().unwrap()));
+    }
 }
