@@ -34,6 +34,8 @@
 //! ```
 use crate::fields::{MultiArch, Priority};
 use crate::lossless::relations::Relations;
+use deb822_lossless::{Deb822, Paragraph};
+use rowan::ast::AstNode;
 
 fn format_field(name: &str, value: &str) -> String {
     match name {
@@ -63,24 +65,35 @@ fn format_field(name: &str, value: &str) -> String {
 }
 
 /// A Debian control file
-pub struct Control(deb822_lossless::Deb822);
+#[derive(Debug, Clone, PartialEq, Eq)]
+pub struct Control(Deb822);
 
 impl Control {
     /// Create a new control file
     pub fn new() -> Self {
-        Control(deb822_lossless::Deb822::new())
+        Control(Deb822::new())
     }
 
     /// Return the underlying deb822 object, mutable
-    pub fn as_mut_deb822(&mut self) -> &mut deb822_lossless::Deb822 {
+    pub fn as_mut_deb822(&mut self) -> &mut Deb822 {
         &mut self.0
     }
 
     /// Return the underlying deb822 object
-    pub fn as_deb822(&self) -> &deb822_lossless::Deb822 {
+    pub fn as_deb822(&self) -> &Deb822 {
         &self.0
     }
 
+    /// Parse control file text, returning a Parse result
+    pub fn parse(text: &str) -> deb822_lossless::Parse<Control> {
+        let deb822_parse = Deb822::parse(text);
+        // Transform Parse<Deb822> to Parse<Control>
+        let green = deb822_parse.green().clone();
+        let errors = deb822_parse.errors().to_vec();
+        let positioned_errors = deb822_parse.positioned_errors().to_vec();
+        deb822_lossless::Parse::new_with_positioned_errors(green, errors, positioned_errors)
+    }
+
     /// Return the source package
     pub fn source(&self) -> Option<Source> {
         self.0
@@ -141,27 +154,27 @@ impl Control {
 
     /// Read a control file from a file
     pub fn from_file<P: AsRef<std::path::Path>>(path: P) -> Result<Self, deb822_lossless::Error> {
-        Ok(Control(deb822_lossless::Deb822::from_file(path)?))
+        Ok(Control(Deb822::from_file(path)?))
     }
 
     /// Read a control file from a file, allowing syntax errors
     pub fn from_file_relaxed<P: AsRef<std::path::Path>>(
         path: P,
     ) -> Result<(Self, Vec<String>), std::io::Error> {
-        let (control, errors) = deb822_lossless::Deb822::from_file_relaxed(path)?;
+        let (control, errors) = Deb822::from_file_relaxed(path)?;
         Ok((Control(control), errors))
     }
 
     /// Read a control file from a reader
     pub fn read<R: std::io::Read>(mut r: R) -> Result<Self, deb822_lossless::Error> {
-        Ok(Control(deb822_lossless::Deb822::read(&mut r)?))
+        Ok(Control(Deb822::read(&mut r)?))
     }
 
     /// Read a control file from a reader, allowing syntax errors
     pub fn read_relaxed<R: std::io::Read>(
         mut r: R,
     ) -> Result<(Self, Vec<String>), deb822_lossless::Error> {
-        let (control, errors) = deb822_lossless::Deb822::read_relaxed(&mut r)?;
+        let (control, errors) = Deb822::read_relaxed(&mut r)?;
         Ok((Self(control), errors))
     }
 
@@ -177,9 +190,7 @@ impl Control {
         immediate_empty_line: bool,
         max_line_length_one_liner: Option<usize>,
     ) {
-        let sort_paragraphs = |a: &deb822_lossless::Paragraph,
-                               b: &deb822_lossless::Paragraph|
-         -> std::cmp::Ordering {
+        let sort_paragraphs = |a: &Paragraph, b: &Paragraph| -> std::cmp::Ordering {
             // Sort Source before Package
             let a_is_source = a.get("Source").is_some();
             let b_is_source = b.get("Source").is_some();
@@ -195,7 +206,7 @@ impl Control {
             a.get("Package").cmp(&b.get("Package"))
         };
 
-        let wrap_paragraph = |p: &deb822_lossless::Paragraph| -> deb822_lossless::Paragraph {
+        let wrap_paragraph = |p: &Paragraph| -> Paragraph {
             // TODO: Add Source/Package specific wrapping
             // TODO: Add support for wrapping and sorting fields
             p.wrap_and_sort(
@@ -211,16 +222,58 @@ impl Control {
             .0
             .wrap_and_sort(Some(&sort_paragraphs), Some(&wrap_paragraph));
     }
+
+    /// Iterate over fields that overlap with the given range
+    ///
+    /// This method returns all fields (entries) from all paragraphs that have any overlap
+    /// with the specified text range. This is useful for incremental parsing in LSP contexts
+    /// where you only want to process fields that were affected by a text change.
+    ///
+    /// # Arguments
+    /// * `range` - The text range to check for overlaps
+    ///
+    /// # Returns
+    /// An iterator over all Entry items that overlap with the given range
+    ///
+    /// # Example
+    /// ```rust
+    /// use debian_control::lossless::Control;
+    /// use deb822_lossless::TextRange;
+    ///
+    /// let control_text = "Source: foo\nMaintainer: test@example.com\n\nPackage: bar\nArchitecture: all\n";
+    /// let control: Control = control_text.parse().unwrap();
+    ///
+    /// // Get fields in a specific range (e.g., where a change occurred)
+    /// let change_range = TextRange::new(20.into(), 40.into());
+    /// for entry in control.fields_in_range(change_range) {
+    ///     if let Some(key) = entry.key() {
+    ///         println!("Field {} was in the changed range", key);
+    ///     }
+    /// }
+    /// ```
+    pub fn fields_in_range(
+        &self,
+        range: rowan::TextRange,
+    ) -> impl Iterator<Item = deb822_lossless::Entry> + '_ {
+        self.0
+            .paragraphs()
+            .flat_map(move |p| p.entries().collect::<Vec<_>>())
+            .filter(move |entry| {
+                let entry_range = entry.syntax().text_range();
+                // Check if ranges overlap
+                entry_range.start() < range.end() && range.start() < entry_range.end()
+            })
+    }
 }
 
-impl From<Control> for deb822_lossless::Deb822 {
+impl From<Control> for Deb822 {
     fn from(c: Control) -> Self {
         c.0
     }
 }
 
-impl From<deb822_lossless::Deb822> for Control {
-    fn from(d: deb822_lossless::Deb822) -> Self {
+impl From<Deb822> for Control {
+    fn from(d: Deb822) -> Self {
         Control(d)
     }
 }
@@ -235,21 +288,22 @@ impl std::str::FromStr for Control {
     type Err = deb822_lossless::ParseError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        Ok(Control(s.parse()?))
+        Control::parse(s).to_result()
     }
 }
 
 /// A source package paragraph
-pub struct Source(deb822_lossless::Paragraph);
+#[derive(Debug, Clone, PartialEq, Eq)]
+pub struct Source(Paragraph);
 
-impl From<Source> for deb822_lossless::Paragraph {
+impl From<Source> for Paragraph {
     fn from(s: Source) -> Self {
         s.0
     }
 }
 
-impl From<deb822_lossless::Paragraph> for Source {
-    fn from(p: deb822_lossless::Paragraph) -> Self {
+impl From<Paragraph> for Source {
+    fn from(p: Paragraph) -> Self {
         Source(p)
     }
 }
@@ -283,12 +337,12 @@ impl Source {
     }
 
     /// Return the underlying deb822 paragraph, mutable
-    pub fn as_mut_deb822(&mut self) -> &mut deb822_lossless::Paragraph {
+    pub fn as_mut_deb822(&mut self) -> &mut Paragraph {
         &mut self.0
     }
 
     /// Return the underlying deb822 paragraph
-    pub fn as_deb822(&self) -> &deb822_lossless::Paragraph {
+    pub fn as_deb822(&self) -> &Paragraph {
         &self.0
     }
 
@@ -572,12 +626,56 @@ impl Source {
     pub fn set_testsuite(&mut self, testsuite: &str) {
         self.0.set("Testsuite", testsuite);
     }
+
+    /// Check if this source paragraph's range overlaps with the given range
+    ///
+    /// # Arguments
+    /// * `range` - The text range to check for overlap
+    ///
+    /// # Returns
+    /// `true` if the paragraph overlaps with the given range, `false` otherwise
+    pub fn overlaps_range(&self, range: rowan::TextRange) -> bool {
+        let para_range = self.0.syntax().text_range();
+        para_range.start() < range.end() && range.start() < para_range.end()
+    }
+
+    /// Get fields in this source paragraph that overlap with the given range
+    ///
+    /// # Arguments
+    /// * `range` - The text range to check for overlaps
+    ///
+    /// # Returns
+    /// An iterator over Entry items that overlap with the given range
+    pub fn fields_in_range(
+        &self,
+        range: rowan::TextRange,
+    ) -> impl Iterator<Item = deb822_lossless::Entry> + '_ {
+        self.0.entries().filter(move |entry| {
+            let entry_range = entry.syntax().text_range();
+            entry_range.start() < range.end() && range.start() < entry_range.end()
+        })
+    }
 }
 
 #[cfg(feature = "python-debian")]
-impl pyo3::ToPyObject for Source {
-    fn to_object(&self, py: pyo3::Python) -> pyo3::PyObject {
-        self.0.to_object(py)
+impl<'py> pyo3::IntoPyObject<'py> for Source {
+    type Target = pyo3::PyAny;
+    type Output = pyo3::Bound<'py, Self::Target>;
+    type Error = pyo3::PyErr;
+
+    fn into_pyobject(self, py: pyo3::Python<'py>) -> Result<Self::Output, Self::Error> {
+        self.0.into_pyobject(py)
+    }
+}
+
+#[cfg(feature = "python-debian")]
+impl<'a, 'py> pyo3::IntoPyObject<'py> for &'a Source {
+    type Target = pyo3::PyAny;
+    type Output = pyo3::Bound<'py, Self::Target>;
+    type Error = pyo3::PyErr;
+
+    fn into_pyobject(self, py: pyo3::Python<'py>) -> Result<Self::Output, Self::Error> {
+        (&self.0).into_pyobject(py)
     }
 }
 
@@ -595,25 +693,57 @@ impl std::fmt::Display for Control {
     }
 }
 
-/// A binary package paragraph
-pub struct Binary(deb822_lossless::Paragraph);
+impl AstNode for Control {
+    type Language = deb822_lossless::Lang;
 
-impl From<Binary> for deb822_lossless::Paragraph {
+    fn can_cast(kind: <Self::Language as rowan::Language>::Kind) -> bool {
+        Deb822::can_cast(kind)
+    }
+
+    fn cast(syntax: rowan::SyntaxNode<Self::Language>) -> Option<Self> {
+        Deb822::cast(syntax).map(Control)
+    }
+
+    fn syntax(&self) -> &rowan::SyntaxNode<Self::Language> {
+        self.0.syntax()
+    }
+}
+
+/// A binary package paragraph
+#[derive(Debug, Clone, PartialEq, Eq)]
+pub struct Binary(Paragraph);
+
+impl From<Binary> for Paragraph {
     fn from(b: Binary) -> Self {
         b.0
     }
 }
 
-impl From<deb822_lossless::Paragraph> for Binary {
-    fn from(p: deb822_lossless::Paragraph) -> Self {
+impl From<Paragraph> for Binary {
+    fn from(p: Paragraph) -> Self {
         Binary(p)
     }
 }
 
 #[cfg(feature = "python-debian")]
-impl pyo3::ToPyObject for Binary {
-    fn to_object(&self, py: pyo3::Python) -> pyo3::PyObject {
-        self.0.to_object(py)
+impl<'py> pyo3::IntoPyObject<'py> for Binary {
+    type Target = pyo3::PyAny;
+    type Output = pyo3::Bound<'py, Self::Target>;
+    type Error = pyo3::PyErr;
+
+    fn into_pyobject(self, py: pyo3::Python<'py>) -> Result<Self::Output, Self::Error> {
+        self.0.into_pyobject(py)
+    }
+}
+
+#[cfg(feature = "python-debian")]
+impl<'a, 'py> pyo3::IntoPyObject<'py> for &'a Binary {
+    type Target = pyo3::PyAny;
+    type Output = pyo3::Bound<'py, Self::Target>;
+    type Error = pyo3::PyErr;
+
+    fn into_pyobject(self, py: pyo3::Python<'py>) -> Result<Self::Output, Self::Error> {
+        (&self.0).into_pyobject(py)
     }
 }
 
@@ -634,16 +764,16 @@ impl Default for Binary {
 impl Binary {
     /// Create a new binary package control file
     pub fn new() -> Self {
-        Binary(deb822_lossless::Paragraph::new())
+        Binary(Paragraph::new())
     }
 
     /// Return the underlying deb822 paragraph, mutable
-    pub fn as_mut_deb822(&mut self) -> &mut deb822_lossless::Paragraph {
+    pub fn as_mut_deb822(&mut self) -> &mut Paragraph {
         &mut self.0
     }
 
     /// Return the underlying deb822 paragraph
-    pub fn as_deb822(&self) -> &deb822_lossless::Paragraph {
+    pub fn as_deb822(&self) -> &Paragraph {
         &self.0
     }
 
@@ -906,6 +1036,35 @@ impl Binary {
     pub fn set_homepage(&mut self, url: &url::Url) {
         self.0.set("Homepage", url.as_str());
     }
+
+    /// Check if this binary paragraph's range overlaps with the given range
+    ///
+    /// # Arguments
+    /// * `range` - The text range to check for overlap
+    ///
+    /// # Returns
+    /// `true` if the paragraph overlaps with the given range, `false` otherwise
+    pub fn overlaps_range(&self, range: rowan::TextRange) -> bool {
+        let para_range = self.0.syntax().text_range();
+        para_range.start() < range.end() && range.start() < para_range.end()
+    }
+
+    /// Get fields in this binary paragraph that overlap with the given range
+    ///
+    /// # Arguments
+    /// * `range` - The text range to check for overlaps
+    ///
+    /// # Returns
+    /// An iterator over Entry items that overlap with the given range
+    pub fn fields_in_range(
+        &self,
+        range: rowan::TextRange,
+    ) -> impl Iterator<Item = deb822_lossless::Entry> + '_ {
+        self.0.entries().filter(move |entry| {
+            let entry_range = entry.syntax().text_range();
+            entry_range.start() < range.end() && range.start() < entry_range.end()
+        })
+    }
 }
 
 #[cfg(test)]
@@ -989,7 +1148,7 @@ Description: this is the short description
     #[test]
     fn test_as_deb822() {
         let control = Control::new();
-        let _deb822: &deb822_lossless::Deb822 = control.as_deb822();
+        let _deb822: &Deb822 = control.as_deb822();
     }
 
     #[test]
@@ -1042,4 +1201,342 @@ Depends: bar (<= 1.0.0), foo
         .to_owned();
         assert_eq!(control.to_string(), expected);
     }
+
+    #[test]
+    fn test_source_wrap_and_sort() {
+        let control: Control = r#"Source: blah
+Build-Depends: foo, bar (>= 1.0.0)
+
+"#
+        .parse()
+        .unwrap();
+        let mut source = control.source().unwrap();
+        source.wrap_and_sort(deb822_lossless::Indentation::Spaces(2), true, None);
+        // The actual behavior - the method modifies the source in-place
+        // but doesn't automatically affect the overall control structure
+        // So we just test that the method executes without error
+        assert!(source.build_depends().is_some());
+    }
+
+    #[test]
+    fn test_binary_set_breaks() {
+        let mut control = Control::new();
+        let mut binary = control.add_binary("foo");
+        let relations: Relations = "bar (>= 1.0.0)".parse().unwrap();
+        binary.set_breaks(Some(&relations));
+        assert!(binary.breaks().is_some());
+    }
+
+    #[test]
+    fn test_binary_set_pre_depends() {
+        let mut control = Control::new();
+        let mut binary = control.add_binary("foo");
+        let relations: Relations = "bar (>= 1.0.0)".parse().unwrap();
+        binary.set_pre_depends(Some(&relations));
+        assert!(binary.pre_depends().is_some());
+    }
+
+    #[test]
+    fn test_binary_set_provides() {
+        let mut control = Control::new();
+        let mut binary = control.add_binary("foo");
+        let relations: Relations = "bar (>= 1.0.0)".parse().unwrap();
+        binary.set_provides(Some(&relations));
+        assert!(binary.provides().is_some());
+    }
+
+    #[test]
+    fn test_source_build_conflicts() {
+        let control: Control = r#"Source: blah
+Build-Conflicts: foo, bar (>= 1.0.0)
+
+"#
+        .parse()
+        .unwrap();
+        let source = control.source().unwrap();
+        let conflicts = source.build_conflicts();
+        assert!(conflicts.is_some());
+    }
+
+    #[test]
+    fn test_source_vcs_svn() {
+        let control: Control = r#"Source: blah
+Vcs-Svn: https://example.com/svn/repo
+
+"#
+        .parse()
+        .unwrap();
+        let source = control.source().unwrap();
+        assert_eq!(
+            source.vcs_svn(),
+            Some("https://example.com/svn/repo".to_string())
+        );
+    }
+
+    #[test]
+    fn test_control_from_conversion() {
+        let deb822_data = r#"Source: test
+Section: libs
+
+"#;
+        let deb822: Deb822 = deb822_data.parse().unwrap();
+        let control = Control::from(deb822);
+        assert!(control.source().is_some());
+    }
+
+    #[test]
+    fn test_fields_in_range() {
+        let control_text = r#"Source: test-package
+Maintainer: Test User <test@example.com>
+Build-Depends: debhelper (>= 12)
+
+Package: test-binary
+Architecture: any
+Depends: ${shlibs:Depends}
+Description: Test package
+ This is a test package
+"#;
+        let control: Control = control_text.parse().unwrap();
+
+        // Test range that covers only the Source field
+        let source_start = 0;
+        let source_end = "Source: test-package".len();
+        let source_range =
+            rowan::TextRange::new((source_start as u32).into(), (source_end as u32).into());
+
+        let fields: Vec<_> = control.fields_in_range(source_range).collect();
+        assert_eq!(fields.len(), 1);
+        assert_eq!(fields[0].key(), Some("Source".to_string()));
+
+        // Test range that covers multiple fields in source paragraph
+        let maintainer_start = control_text.find("Maintainer:").unwrap();
+        let build_depends_end = control_text
+            .find("Build-Depends: debhelper (>= 12)")
+            .unwrap()
+            + "Build-Depends: debhelper (>= 12)".len();
+        let multi_range = rowan::TextRange::new(
+            (maintainer_start as u32).into(),
+            (build_depends_end as u32).into(),
+        );
+
+        let fields: Vec<_> = control.fields_in_range(multi_range).collect();
+        assert_eq!(fields.len(), 2);
+        assert_eq!(fields[0].key(), Some("Maintainer".to_string()));
+        assert_eq!(fields[1].key(), Some("Build-Depends".to_string()));
+
+        // Test range that spans across paragraphs
+        let cross_para_start = control_text.find("Build-Depends:").unwrap();
+        let cross_para_end =
+            control_text.find("Architecture: any").unwrap() + "Architecture: any".len();
+        let cross_range = rowan::TextRange::new(
+            (cross_para_start as u32).into(),
+            (cross_para_end as u32).into(),
+        );
+
+        let fields: Vec<_> = control.fields_in_range(cross_range).collect();
+        assert_eq!(fields.len(), 3); // Build-Depends, Package, Architecture
+        assert_eq!(fields[0].key(), Some("Build-Depends".to_string()));
+        assert_eq!(fields[1].key(), Some("Package".to_string()));
+        assert_eq!(fields[2].key(), Some("Architecture".to_string()));
+
+        // Test empty range (should return no fields)
+        let empty_range = rowan::TextRange::new(1000.into(), 1001.into());
+        let fields: Vec<_> = control.fields_in_range(empty_range).collect();
+        assert_eq!(fields.len(), 0);
+    }
+
+    #[test]
+    fn test_source_overlaps_range() {
+        let control_text = r#"Source: test-package
+Maintainer: Test User <test@example.com>
+
+Package: test-binary
+Architecture: any
+"#;
+        let control: Control = control_text.parse().unwrap();
+        let source = control.source().unwrap();
+
+        // Test range that overlaps with source paragraph
+        let overlap_range = rowan::TextRange::new(10.into(), 30.into());
+        assert!(source.overlaps_range(overlap_range));
+
+        // Test range that doesn't overlap with source paragraph
+        let binary_start = control_text.find("Package:").unwrap();
+        let no_overlap_range = rowan::TextRange::new(
+            (binary_start as u32).into(),
+            ((binary_start + 20) as u32).into(),
+        );
+        assert!(!source.overlaps_range(no_overlap_range));
+
+        // Test range that starts before and ends within source paragraph
+        let partial_overlap = rowan::TextRange::new(0.into(), 15.into());
+        assert!(source.overlaps_range(partial_overlap));
+    }
+
+    #[test]
+    fn test_source_fields_in_range() {
+        let control_text = r#"Source: test-package
+Maintainer: Test User <test@example.com>
+Build-Depends: debhelper (>= 12)
+
+Package: test-binary
+"#;
+        let control: Control = control_text.parse().unwrap();
+        let source = control.source().unwrap();
+
+        // Test range covering Maintainer field
+        let maintainer_start = control_text.find("Maintainer:").unwrap();
+        let maintainer_end = maintainer_start + "Maintainer: Test User <test@example.com>".len();
+        let maintainer_range = rowan::TextRange::new(
+            (maintainer_start as u32).into(),
+            (maintainer_end as u32).into(),
+        );
+
+        let fields: Vec<_> = source.fields_in_range(maintainer_range).collect();
+        assert_eq!(fields.len(), 1);
+        assert_eq!(fields[0].key(), Some("Maintainer".to_string()));
+
+        // Test range covering multiple fields
+        let all_source_range = rowan::TextRange::new(0.into(), 100.into());
+        let fields: Vec<_> = source.fields_in_range(all_source_range).collect();
+        assert_eq!(fields.len(), 3); // Source, Maintainer, Build-Depends
+    }
+
+    #[test]
+    fn test_binary_overlaps_range() {
+        let control_text = r#"Source: test-package
+
+Package: test-binary
+Architecture: any
+Depends: ${shlibs:Depends}
+"#;
+        let control: Control = control_text.parse().unwrap();
+        let binary = control.binaries().next().unwrap();
+
+        // Test range that overlaps with binary paragraph
+        let package_start = control_text.find("Package:").unwrap();
+        let overlap_range = rowan::TextRange::new(
+            (package_start as u32).into(),
+            ((package_start + 30) as u32).into(),
+        );
+        assert!(binary.overlaps_range(overlap_range));
+
+        // Test range before binary paragraph
+        let no_overlap_range = rowan::TextRange::new(0.into(), 10.into());
+        assert!(!binary.overlaps_range(no_overlap_range));
+    }
+
+    #[test]
+    fn test_binary_fields_in_range() {
+        let control_text = r#"Source: test-package
+
+Package: test-binary
+Architecture: any
+Depends: ${shlibs:Depends}
+Description: Test binary
+ This is a test binary package
+"#;
+        let control: Control = control_text.parse().unwrap();
+        let binary = control.binaries().next().unwrap();
+
+        // Test range covering Architecture and Depends
+        let arch_start = control_text.find("Architecture:").unwrap();
+        let depends_end = control_text.find("Depends: ${shlibs:Depends}").unwrap()
+            + "Depends: ${shlibs:Depends}".len();
+        let range = rowan::TextRange::new((arch_start as u32).into(), (depends_end as u32).into());
+
+        let fields: Vec<_> = binary.fields_in_range(range).collect();
+        assert_eq!(fields.len(), 2);
+        assert_eq!(fields[0].key(), Some("Architecture".to_string()));
+        assert_eq!(fields[1].key(), Some("Depends".to_string()));
+
+        // Test partial overlap with Description field
+        let desc_start = control_text.find("Description:").unwrap();
+        let partial_range = rowan::TextRange::new(
+            ((desc_start + 5) as u32).into(),
+            ((desc_start + 15) as u32).into(),
+        );
+        let fields: Vec<_> = binary.fields_in_range(partial_range).collect();
+        assert_eq!(fields.len(), 1);
+        assert_eq!(fields[0].key(), Some("Description".to_string()));
+    }
+
+    #[test]
+    fn test_incremental_parsing_use_case() {
+        // This test simulates a real LSP use case where only changed fields are processed
+        let control_text = r#"Source: example
+Maintainer: John Doe <john@example.com>
+Standards-Version: 4.6.0
+Build-Depends: debhelper-compat (= 13)
+
+Package: example-bin
+Architecture: all
+Depends: ${misc:Depends}
+Description: Example package
+ This is an example.
+"#;
+        let control: Control = control_text.parse().unwrap();
+
+        // Simulate a change to Standards-Version field
+        let change_start = control_text.find("Standards-Version:").unwrap();
+        let change_end = change_start + "Standards-Version: 4.6.0".len();
+        let change_range =
+            rowan::TextRange::new((change_start as u32).into(), (change_end as u32).into());
+
+        // Only process fields in the changed range
+        let affected_fields: Vec<_> = control.fields_in_range(change_range).collect();
+        assert_eq!(affected_fields.len(), 1);
+        assert_eq!(
+            affected_fields[0].key(),
+            Some("Standards-Version".to_string())
+        );
+
+        // Verify that we're not processing unrelated fields
+        for entry in &affected_fields {
+            let key = entry.key().unwrap();
+            assert_ne!(key, "Maintainer");
+            assert_ne!(key, "Build-Depends");
+            assert_ne!(key, "Architecture");
+        }
+    }
+
+    #[test]
+    fn test_positioned_parse_errors() {
+        // Test case from the requirements document
+        let input = "Invalid: field\nBroken field without colon";
+        let parsed = Control::parse(input);
+
+        // Should have positioned errors accessible
+        let positioned_errors = parsed.positioned_errors();
+        assert!(
+            !positioned_errors.is_empty(),
+            "Should have positioned errors"
+        );
+
+        // Test that we can access error properties
+        for error in positioned_errors {
+            let start_offset: u32 = error.range.start().into();
+            let end_offset: u32 = error.range.end().into();
+
+            // Verify we have meaningful error messages
+            assert!(!error.message.is_empty());
+
+            // Verify ranges are valid
+            assert!(start_offset <= end_offset);
+            assert!(end_offset <= input.len() as u32);
+
+            // Error should have a code
+            assert!(error.code.is_some());
+
+            println!(
+                "Error at {:?}: {} (code: {:?})",
+                error.range, error.message, error.code
+            );
+        }
+
+        // Should also be able to get string errors for backward compatibility
+        let string_errors = parsed.errors();
+        assert!(!string_errors.is_empty());
+        assert_eq!(string_errors.len(), positioned_errors.len());
+    }
 }
